package org.tekkotsu.ui.storyboard.objects;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.ImageData;
import org.tekkotsu.ui.editor.model.StateNodeModel;
import org.tekkotsu.ui.storyboard.model.AbstractModel;
import org.tekkotsu.ui.storyboard.model.LogModel;
import org.tekkotsu.ui.storyboard.panels.TimeLine;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;

/**
 * @author asangpet
 * 
 */
public class LogViewObject extends AbstractViewObject {
	LogModel model;

	StateNodeModel state;

	public LogViewObject(LogModel model, StateNodeModel state,
			StoryboardViewer viewer) {
		super(viewer);
		setState(state);
		setModel(model);
		setToolTip(getToolTipFigure());
	}

	private IFigure getToolTipFigure() {		
		Label label = new Label(model.getETIDString() + "\n" + model.getRuntimeViewData().toStringExtended());
		if (model.getETID().equals(AbstractModel.ETID_ACTIVATE)) {
			label.setForegroundColor(ColorConstants.darkGreen);
		} else if (model.getETID().equals(AbstractModel.ETID_DEACTIVATE)) {
			label.setForegroundColor(ColorConstants.red);
		} else if (model.getETID().equals(AbstractModel.ETID_STATUS)) {
			label.setForegroundColor(ColorConstants.blue);
		}
		return label;
	}

	private void setModel(LogModel model) {
		this.model = model;
		Rectangle bound = null;
		if (state == null)
			bound = new Rectangle(0, 0, 10, TimeLine.HEIGHT);
		else
			bound = state.getConstraint();

		int xoff = timeline.getOffsetOfTime(model.getStart());

		ImageData data;

		if (imgReg.get(model.getIconName()) == null) {
			ImageDescriptor desc = ImageDescriptor.createFromFile(
					org.tekkotsu.ui.storyboard.icons.IconDummy.class, model
							.getIconName());

			data = desc.getImageData();
			
			// fallback to known icon
			if( data == null ) {
				desc = ImageDescriptor.createFromFile(
						org.tekkotsu.ui.storyboard.icons.IconDummy.class, "__fallback__.png");
				data = desc.getImageData();
			}

			imgReg.put(model.getIconName(), desc);
		} else {
			data = imgReg.getDescriptor(model.getIconName()).getImageData();
		}
		setConstraint(xoff - data.width / 2, bound.y
				+ (bound.height - data.height) / 2, data.width, data.height);
	}

	@Override
	protected void paintFigure(Graphics g2d) {
		Rectangle clArea = getClientArea();
		Rectangle bound = getConstraint();
		
		g2d.drawImage(imgReg.get(model.getIconName()), new Point(clArea.x,
				clArea.y));

		if(model.isSelected()) {
			if (model.getETID().equals(AbstractModel.ETID_DEACTIVATE)) {
				g2d.setForegroundColor(ColorConstants.red);
				g2d.setLineWidth(3);
				g2d.drawLine(clArea.x, clArea.y, clArea.x + bound.width, clArea.y
						+ bound.height);
			} else if (model.getETID().equals(AbstractModel.ETID_STATUS)) {
				g2d.setForegroundColor(ColorConstants.blue);
				g2d.setLineWidth(3);
				g2d.drawLine(clArea.x, clArea.y + bound.height , clArea.x
						+ bound.width, clArea.y + bound.height );
			} else if (model.getETID().equals(AbstractModel.ETID_ACTIVATE)) {
				g2d.setForegroundColor(ColorConstants.green);
				g2d.setLineWidth(3);
				g2d.drawLine(clArea.x, clArea.y + bound.height , clArea.x
						+ bound.width, clArea.y + bound.height );
			}
		}
	}

	/**
	 * @return Returns the model.
	 */
	@Override
	public LogModel getModel() {
		return model;
	}

	private void setState(StateNodeModel state) {
		this.state = state;
	}
	
	@Override
	public String getID() {
		return model.getID();
	}

	@Override
	public void updateColors() {
		// nothing
	}
}