package org.tekkotsu.ui.storyboard.views;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;
import org.tekkotsu.ui.storyboard.model.AbstractModel;
import org.tekkotsu.ui.storyboard.panels.AbstractTimePanel;

public class RuntimeView extends ViewPart {	
	public static NumberFormat nf = new DecimalFormat();
	static {
		nf.setMaximumFractionDigits(4);
		nf.setMinimumFractionDigits(4);
	}
	public static String timeToString(int time) {
		if( time == AbstractModel.UNKNOWN_TIME ) {
			return "unknown";
		}
		return nf.format(time*AbstractTimePanel.timeScale) + "s";
	}
	
	private TreeViewer viewer;
	
	
	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It can wrap
	 * existing objects in adapters or simply return
	 * objects as-is. These objects may be sensitive
	 * to the current input of the view, or ignore
	 * it and always show the same content 
	 * (like Task List, for example).
	 */
	 
	private ViewContentProvider content;


	private TreeParent root;
	private int currentRootTime;

	static class ViewLabelProvider extends LabelProvider {
		@Override
		public String getText(Object obj) {
			return obj.toString();
		}
		@Override
		public Image getImage(Object obj) {
			return null;
		}
	}
	static class NameSorter extends ViewerSorter {
	    @Override
		public int compare(Viewer viewer, Object e1, Object e2) {
	        int cat1 = category(e1);
	        int cat2 = category(e2);

	        if (cat1 != cat2)
	            return cat1 - cat2;
	        if (e1 instanceof TreeParent) {
	        	TreeParent t1 = (TreeParent)e1;
	        	TreeParent t2 = (TreeParent)e2;
	        	int result = t1.getStop()-t2.getStop();
	        	if (result!=0) return result;
	        	result = t1.getStart()-t2.getStart();
	        	if (result!=0) return result;
	        }
	        return super.compare(viewer,e1,e2);
	    }
	}

	/**
	 * The constructor.
	 */

	private static RuntimeView selfInstance;
	
	public static RuntimeView getInstance()
	{
		return selfInstance;
	}
	
	public RuntimeView() {
		selfInstance = this;
	}

	public void setContent(TreeParent root) {
		this.root = root;
		content.setContent(root);
		
		viewer.getControl().getDisplay().syncExec(new Runnable() {			
			public void run() {
				viewer.setContentProvider(content);						
				viewer.expandToLevel(3);
			}
		});
	}
	
	public TreeParent getContent(int time) {
		if( currentRootTime == time ) {
			return root;
		}
		
		root = new TreeParent("Current selection :" + timeToString(time));
		currentRootTime = time;
		return root;
	}
	
	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		content = new ViewContentProvider();
		viewer.setContentProvider(content);
		viewer.setLabelProvider(new ViewLabelProvider());
		//viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	public void updateSelection(List<Object> selection)
	{
		//content.setContent(selection);
		viewer.getControl().getDisplay().syncExec(new Runnable() {			
			public void run() {
				viewer.setContentProvider(content);						
				viewer.expandToLevel(3);
			}
		});
	}

	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It can wrap
	 * existing objects in adapters or simply return
	 * objects as-is. These objects may be sensitive
	 * to the current input of the view, or ignore
	 * it and always show the same content 
	 * (like Task List, for example).
	 */
	
	public static class TreeObject implements IAdaptable {
		private String name;
		private TreeParent parent;
		
		public TreeObject(String name) {
			this.name = name;
		}
		public String getName() {
			return name;
		}
		public void setParent(TreeParent parent) {
			this.parent = parent;
		}
		public TreeParent getParent() {
			return parent;
		}
		@Override
		public String toString() {
			return getName();
		}
		@SuppressWarnings("unchecked")
		public Object getAdapter(Class key) {
			return null;
		}
		public String toStringExtended() {
			return toString();
		}
	}

	public static class TreeParent extends TreeObject {
		private ArrayList<TreeObject> children;
		private int start,stop;
		public int getStop() {
			return stop;
		}
		public int getStart() {
			return start;
		}
		public TreeParent(String name, int start,int stop) {
			super(name);
			this.start = start; this.stop = stop;
			children = new ArrayList<TreeObject>();
		}
		public TreeParent(String name) {
			this(name,0,0);
		}
	
		public void addChild(TreeObject child) {
			children.add(child);
			child.setParent(this);
		}
		public void removeChild(TreeObject child) {
			children.remove(child);
			child.setParent(null);
		}
		public TreeObject [] getChildren() {
			return children.toArray(new TreeObject[children.size()]);
		}
		public boolean hasChildren() {
			return children.size()>0;
		}
		
		@Override
		public String toStringExtended() {
			StringBuffer sb = new StringBuffer();
			sb.append(super.toString());
			
			for( TreeObject child : children ) {
				sb.append("\n");
				sb.append(child.toString());
			}
			return sb.toString();
		}
	}

	private class ViewContentProvider implements ITreeContentProvider {
		private TreeParent invisibleRoot;
	
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			//nothing
		}
		public void dispose() {
			//nothing
		}
		public Object[] getElements(Object parent) {
			if (parent.equals(getViewSite())) {
				if (invisibleRoot==null) initialize();
				return getChildren(invisibleRoot);
			}
			return getChildren(parent);
		}
		public Object getParent(Object child) {
			if (child instanceof TreeObject) {
				return ((TreeObject)child).getParent();
			}
			return null;
		}
		public Object [] getChildren(Object parent) {
			if (parent instanceof TreeParent) {
				return ((TreeParent)parent).getChildren();
			}
			return new Object[0];
		}
		public boolean hasChildren(Object parent) {
			if (parent instanceof TreeParent)
				return ((TreeParent)parent).hasChildren();
			return false;
		}
	
		private void initialize() {
			invisibleRoot = new TreeParent("");
		}
		
		public void setContent(TreeParent root) {
			invisibleRoot = new TreeParent("");
			invisibleRoot.addChild(root);
		}
	}

	public void clear() {
		getContent(-1);
	}
}